@@ -658,6 +658,78 @@ class TestCredentials(object):
 
         assert excinfo.match(r"missing 'email' field")
 
+    @mock.patch("google.auth.compute_engine._metadata.get")
+    @mock.patch("google.auth._agent_identity_utils.get_agent_identity_certificate_path")
+    @mock.patch("google.auth._agent_identity_utils.parse_certificate")
+    @mock.patch(
+        "google.auth._agent_identity_utils.should_request_bound_token",
+        return_value=True,
+    )
+    @mock.patch(
+        "google.auth._agent_identity_utils.calculate_certificate_fingerprint",
+        return_value="fingerprint",
+    )
+    def test_refresh_with_agent_identity(
+        self,
+        mock_calculate_fingerprint,
+        mock_should_request,
+        mock_parse_certificate,
+        mock_get_path,
+        mock_metadata_get,
+        tmpdir,
+    ):
+        cert_path = tmpdir.join("cert.pem")
+        cert_path.write(b"cert_content")
+        mock_get_path.return_value = str(cert_path)
+
+        mock_metadata_get.side_effect = [
+            {"email": "service-account@example.com", "scopes": ["one", "two"]},
+            {"access_token": "token", "expires_in": 500},
+        ]
+
+        self.credentials.refresh(None)
+
+        assert self.credentials.token == "token"
+        mock_parse_certificate.assert_called_once_with(b"cert_content")
+        mock_should_request.assert_called_once_with(mock_parse_certificate.return_value)
+        kwargs = mock_metadata_get.call_args[1]
+        assert kwargs["params"] == {
+            "scopes": "one,two",
+            "bindCertificateFingerprint": "fingerprint",
+        }
+
+    @mock.patch("google.auth.compute_engine._metadata.get")
+    @mock.patch("google.auth._agent_identity_utils.get_agent_identity_certificate_path")
+    @mock.patch("google.auth._agent_identity_utils.parse_certificate")
+    @mock.patch(
+        "google.auth._agent_identity_utils.should_request_bound_token",
+        return_value=False,
+    )
+    def test_refresh_with_agent_identity_opt_out_or_not_agent(
+        self,
+        mock_should_request,
+        mock_parse_certificate,
+        mock_get_path,
+        mock_metadata_get,
+        tmpdir,
+    ):
+        cert_path = tmpdir.join("cert.pem")
+        cert_path.write(b"cert_content")
+        mock_get_path.return_value = str(cert_path)
+
+        mock_metadata_get.side_effect = [
+            {"email": "service-account@example.com", "scopes": ["one", "two"]},
+            {"access_token": "token", "expires_in": 500},
+        ]
+
+        self.credentials.refresh(None)
+
+        assert self.credentials.token == "token"
+        mock_parse_certificate.assert_called_once_with(b"cert_content")
+        mock_should_request.assert_called_once_with(mock_parse_certificate.return_value)
+        kwargs = mock_metadata_get.call_args[1]
+        assert "bindCertificateFingerprint" not in kwargs.get("params", {})
+
 
 class TestIDTokenCredentials(object):
     credentials = None
