@@ -506,6 +506,7 @@ class TestCredentials(object):
             quota_project_id=QUOTA_PROJECT_ID,
             workforce_pool_user_project=None,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -535,6 +536,7 @@ class TestCredentials(object):
             quota_project_id=None,
             workforce_pool_user_project=None,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -566,6 +568,7 @@ class TestCredentials(object):
             quota_project_id=None,
             workforce_pool_user_project=None,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -596,6 +599,7 @@ class TestCredentials(object):
             quota_project_id=None,
             workforce_pool_user_project=WORKFORCE_POOL_USER_PROJECT,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -632,6 +636,7 @@ class TestCredentials(object):
             quota_project_id=QUOTA_PROJECT_ID,
             workforce_pool_user_project=None,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -662,6 +667,7 @@ class TestCredentials(object):
             quota_project_id=None,
             workforce_pool_user_project=None,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     @mock.patch.object(identity_pool.Credentials, "__init__", return_value=None)
@@ -693,6 +699,7 @@ class TestCredentials(object):
             quota_project_id=None,
             workforce_pool_user_project=WORKFORCE_POOL_USER_PROJECT,
             universe_domain=DEFAULT_UNIVERSE_DOMAIN,
+            trust_boundary=None,
         )
 
     def test_constructor_nonworkforce_with_workforce_pool_user_project(self):
@@ -1765,3 +1772,59 @@ class TestCredentials(object):
         assert excinfo.match(
             'The credential is not configured to use mtls requests. The credential should include a "certificate" section in the credential source.'
         )
+
+    @mock.patch("google.auth._agent_identity_utils.parse_certificate")
+    @mock.patch(
+        "google.auth._agent_identity_utils.should_request_bound_token",
+        return_value=True,
+    )
+    @mock.patch(
+        "google.auth._agent_identity_utils.calculate_certificate_fingerprint",
+        return_value="fingerprint",
+    )
+    @mock.patch.object(
+        identity_pool.Credentials, "_get_cert_bytes", return_value=b"cert"
+    )
+    @mock.patch.object(external_account.Credentials, "_refresh_token")
+    def test_refresh_with_agent_identity(
+        self,
+        mock_refresh_token,
+        mock_get_cert_bytes,
+        mock_calculate_fingerprint,
+        mock_should_request,
+        mock_parse_certificate,
+    ):
+        mock_parse_certificate.return_value = mock.sentinel.cert
+        credentials = self.make_credentials(
+            credential_source=self.CREDENTIAL_SOURCE_CERTIFICATE.copy()
+        )
+        credentials.refresh(None)
+        mock_parse_certificate.assert_called_once_with(b"cert")
+        mock_should_request.assert_called_once_with(mock.sentinel.cert)
+        mock_calculate_fingerprint.assert_called_once_with(mock.sentinel.cert)
+        mock_refresh_token.assert_called_once_with(None, cert_fingerprint="fingerprint")
+
+    @mock.patch("google.auth._agent_identity_utils.parse_certificate")
+    @mock.patch(
+        "google.auth._agent_identity_utils.should_request_bound_token",
+        return_value=False,
+    )
+    @mock.patch.object(
+        identity_pool.Credentials, "_get_cert_bytes", return_value=b"cert"
+    )
+    @mock.patch.object(external_account.Credentials, "_refresh_token")
+    def test_refresh_with_agent_identity_opt_out_or_not_agent(
+        self,
+        mock_refresh_token,
+        mock_get_cert_bytes,
+        mock_should_request,
+        mock_parse_certificate,
+    ):
+        mock_parse_certificate.return_value = mock.sentinel.cert
+        credentials = self.make_credentials(
+            credential_source=self.CREDENTIAL_SOURCE_CERTIFICATE.copy()
+        )
+        credentials.refresh(None)
+        mock_parse_certificate.assert_called_once_with(b"cert")
+        mock_should_request.assert_called_once_with(mock.sentinel.cert)
+        mock_refresh_token.assert_called_once_with(None, cert_fingerprint=None)
